#ifndef _CDS345_H_
#define _DS345_H_

// Generator output channel. DS345 ONLY SUPPORTS CHANNEL 1
typedef enum {
    CH1 = 1,
    CH2 = 2
} chan_t;

// Output waveform codes used in DS345
typedef enum {
    SINE,
    SQUARE,
    TRIANGLE,
    RAMP,
    NOISE,
    ARBITRARY
} wave_t;

// Class Declaration for DS345 Generator///////////////////////////////////////
class cDS345
{
public:
    // Constructors, Destructors //////////////////////////
    cDS345(/* args */);
    ~cDS345();

    // Other Public Member Functions //////////////////////
    void sendCommand(const char* cmd);
    char* readResponse(void);
    void wave2text(wave_t wave, char** text);
    void reset(void);
    void setWave(wave_t type);
    wave_t getWave(void);
    void setFrequency(double freq);
    double getFrequency(void);
    void setAmplitude(float ampl);
    float getAmplitude(void);
    void setOffset(float offs);
    float getOffset(void);
    void setLoadFactor(unsigned int load);
    size_t getAllParams(char** paramString);

    // Private Member Functions ///////////////////////////
private:
    char* readParam(const char*);
    void readWave(void);
    void readFrequency(void);
    void readAmplitude(void);
    void readOffset(void);

// memmber Variables //////////////////////////////////////
private:
    double  m_fFrequency;   // in Hz

    // These are the effective voltages at the specified load
    float   m_fAmplitude;   // in volts peak-peak
    float   m_fOffset;      // in volts
    
    chan_t  m_Channel;      // always = 1
    wave_t  m_WaveType;
    double  m_fLoadFactor;  // to correct for load impedance
                            // cf. comment @ setLoadFactor()
    static const __SIZE_TYPE__ RESPONSE_LENGTH = 200;
    char    m_sResponse[RESPONSE_LENGTH]; // Response buffer

    static const __SIZE_TYPE__ BUF_SIZE = 100;
    char m_buf[BUF_SIZE];   // General purpose string buffer
};

#endif